/*
 * feedbackd
 *
 * Copyright (c) 2012 - 2017 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#ifndef __DBUS_H__
#define __DBUS_H__

#include <gio/gio.h>
#include <glib-unix.h>
#include <gio/gunixfdlist.h>
#include <stdarg.h>
#include <assert.h>

/*
 * Template
 *
#define XXX_BUS_NAME                        "org.tizen.system.XXX"
#define XXX_OBJECT_PATH                     "/Org/Tizen/System/XXX"
#define XXX_INTERFACE_NAME                  XXX_BUS_NAME
#define XXX_PATH_YYY                        XXX_OBJECT_PATH"/YYY"
#define XXX_INTERFACE_YYY                   XXX_INTERFACE_NAME".YYY"
#define XXX_SIGNAL_ZZZ                      "ZZZ"
#define XXX_METHOD_ZZZ                      "ZZZ"
 */

/*
 * DBus daemon
 */
#define DBUS_BUS_NAME                       "org.freedesktop.DBus"
#define DBUS_OBJECT_PATH                    "/org/freedesktop/DBus"
#define DBUS_INTERFACE_NAME                 DBUS_BUS_NAME

/*
 * Device daemon
 */
#define DEVICED_BUS_NAME                    "org.tizen.system.deviced"
#define DEVICED_OBJECT_PATH                 "/Org/Tizen/System/DeviceD"
#define DEVICED_INTERFACE_NAME              DEVICED_BUS_NAME
/* Core service: get/set device status operations about device */
#define DEVICED_PATH_CORE                   DEVICED_OBJECT_PATH"/Core"
#define DEVICED_INTERFACE_CORE              DEVICED_INTERFACE_NAME".core"
/* Poweroff service: get power off status operations about Poweroff */
#define DEVICED_PATH_POWEROFF               DEVICED_OBJECT_PATH"/PowerOff"
#define DEVICED_INTERFACE_POWEROFF          DEVICED_INTERFACE_NAME".PowerOff"
/* Key service: operations about key */
#define DEVICED_PATH_KEY                    DEVICED_OBJECT_PATH"/Key"
#define DEVICED_INTERFACE_KEY               DEVICED_INTERFACE_NAME".Key"

/*
 * Vibrator daemon
 */
#define VIBRATOR_BUS_NAME                   "org.tizen.system.vibrator"
#define VIBRATOR_OBJECT_PATH                "/Org/Tizen/System/Vibrator"
#define VIBRATOR_INTERFACE_NAME              VIBRATOR_BUS_NAME
/* Core service: get/set device status operations about device */
#define VIBRATOR_PATH_CORE                   VIBRATOR_OBJECT_PATH"/Core"
#define VIBRATOR_INTERFACE_CORE              VIBRATOR_INTERFACE_NAME".core"

#define VIBRATOR_PATH_HAPTIC                 VIBRATOR_OBJECT_PATH"/Haptic"
#define VIBRATOR_INTERFACE_HAPTIC            VIBRATOR_INTERFACE_NAME".haptic"

typedef struct {
	const unsigned char *data;
	int size;
} dbus_byte;

GVariant *dbus_method_sync_with_reply(const char *dest, const char *path,
		const char *interface, const char *method,
		const char *sig, const char *param[]);

GVariant *dbus_method_sync_with_reply_var(const char *dest, const char *path, const char *iface, const char *method, GVariant *var);

/* fd */
gint* dbus_handle_get_unix_fd_list(GDBusMethodInvocation *invocation, int *size);

GVariant *dbus_method_with_unix_fd_list_sync_with_reply(const char *dest, const char *path,
		const char *iface, const char *method,
		const char *signature, const char *param[],
		int *in_fdlist, int in_size,
		int **out_fdlist, int *out_size);
GVariant *dbus_method_with_unix_fd_list_sync_with_reply_var(const char *dest, const char *path,
		const char *iface, const char *method,
		GVariant *param,
		int *in_fdlist, int in_size,
		int **out_fdlist, int *out_size);

int dbus_method_sync(const char *dest, const char *path,
		const char *interface, const char *method,
		const char *sig, const char *param[]);

int dbus_method_sync_var(const char *dest, const char *path, const char *iface, const char *method, GVariant *param);

int dbus_method_sync_timeout(const char *dest, const char *path,
		const char *interface, const char *method,
		const char *sig, const char *param[], int timeout);

int dbus_method_sync_pairs(const char *dest, const char *path,
		const char *interface, const char *method,
		int num, va_list args);
int dbus_method_async_pairs(const char *dest, const char *path,
		const char *interface, const char *method,
		int num, va_list args);
int dbus_method_async(const char *dest, const char *path,
		const char *interface, const char *method,
		const char *sig, const char *param[]);

int dbus_method_async_var(const char *dest, const char *path, const char *iface, const char *method, GVariant *param);

typedef void (*dbus_pending_cb)(GVariant *var, void *user_data, GError *err);

int dbus_method_async_with_reply(const char *dest,
		const char *path,
		const char *iface,
		const char *method,
		const char *signature,
		const char *param[],
		dbus_pending_cb cb,
		int timeout_msec,
		void *data);


int dbus_method_async_with_reply_var(const char *dest,
		const char *path,
		const char *iface,
		const char *method,
		GVariant *param,
		dbus_pending_cb cb,
		int timeout_msec,
		void *data);

int check_systemd_active(void);

/**
 * @brief   Dbus handler which is used to register and call methods

 * @since_tizen 4.0
 */
typedef void *dbus_handle_h;

/**
 * @brief   Dbus method handler which is used to unregister dbus methods
 * @since_tizen 4.0
 */
typedef void *dbus_object_handle_h;

/**
 * @brief   Structure which contains the dbus method name and callback function.
 * @since_tizen 4.0
 */
typedef struct {
	const char *member;
	const char *signature_in;
	const char *signature_out;
	GVariant *(*func)(GDBusConnection *conn,
		const gchar *sender, const gchar *path, const gchar *iface, const gchar *name,
		GVariant *param, GDBusMethodInvocation *invocation, gpointer user_data);
} dbus_method_s;

/**
 * @brief   Structure which contains the dbus interface information and its methods.i
 * @since_tizen 4.0
 */
typedef struct {
	const char *name;
	const dbus_method_s *methods;
	int nr_methods;
} dbus_interface_u;

#define dh_get_param_from_var(gvar, signature, ...) ((g_strcmp0(signature, g_variant_get_type_string(gvar)) == 0) ? g_variant_get(gvar, signature, __VA_ARGS__), TRUE : FALSE)

#define dbus_handle_new_g_variant_tuple() g_variant_new_tuple(NULL, 0)

/**
 * @brief   Callback function which is called when the user data needs to be released.
 * @since_tizen 4.0
 */
typedef void (*destroy_notified)(void *data);

typedef struct {
	dbus_pending_cb func;
	void *data;
} pending_call_data;

int dbus_handle_request_bus_name(dbus_handle_h handle, const char *bus_name);

dbus_handle_h dbus_handle_get_connection(GBusType bus_type, gboolean priv);

int dbus_handle_register_dbus_object(dbus_handle_h handle, const char *obj_path, const dbus_interface_u *iface);
int dbus_handle_unregister_dbus_object(dbus_handle_h handle, const char *obj_path);

int dbus_handle_add_dbus_object(dbus_handle_h handle, const char *obj_path, const dbus_interface_u *iface_u);
int dbus_handle_register_dbus_object_all(dbus_handle_h handle);

guint subscribe_dbus_signal(dbus_handle_h handle, const char *path, const char *iface, const char *name, GDBusSignalCallback cb, void *data, destroy_notified free_func);
void unsubscribe_dbus_signal(dbus_handle_h handle, guint id);

GVariant *dbus_handle_make_simple_array(const char *sig, int *param);

int dbus_handle_broadcast_dbus_signal(const char *path, const char *iface, const char *name, const char *signature, const char *param[]);
int dbus_handle_broadcast_dbus_signal_var(const char *path, const char *iface, const char *name, GVariant *param);

typedef struct
{
	guint   pid;
	guint   uid;
	gchar  *unique_name;
	gchar  *sec_label;
} GDBusCredentials;

int dbus_connection_get_sender_pid(GDBusConnection *conn, const char * sender);
int dbus_handle_get_sender_pid(dbus_handle_h handle, const char * sender);
int dbus_handle_get_sender_credentials(dbus_handle_h handle, const char *name, GDBusCredentials *creds);
int dbus_handle_watch_name(const char *name, GBusNameAppearedCallback name_appeared_handler, GBusNameVanishedCallback name_vanished_handler, void *user_data);
void dbus_handle_unwatch_name(guint id);
dbus_handle_h dbus_handle_init(GBusType type, const char* bus_name);
char** dbus_handle_get_owner_list(dbus_handle_h handle, const char *bus_name);
void dbush_handle_check_owner_name(dbus_handle_h handle, const char *owner_name);

#endif
